"""
aasrp url config
"""

from django.conf.urls import url

from aasrp import views

app_name: str = "aasrp"

urlpatterns = [
    url(r"^$", views.dashboard, name="dashboard"),
    url(r"^all/$", views.dashboard, {"show_all_links": True}, name="all"),
    url(r"^add/$", views.srp_link_add, name="add_srp_link"),
    url(r"^srp-link/(\w+)/edit/$", views.srp_link_edit, name="edit_srp_link"),
    url(
        r"^srp-link/(\w+)/view-srp-requests/$",
        views.srp_link_view_requests,
        name="view_srp_requests",
    ),
    url(r"^srp-link/(\w+)/enable/$", views.enable_srp_link, name="enable_srp_link"),
    url(r"^srp-link/(\w+)/disable/$", views.disable_srp_link, name="disable_srp_link"),
    url(r"^srp-link/(\w+)/delete/$", views.delete_srp_link, name="delete_srp_link"),
    url(r"^srp-link/(\w+)/request-srp/$", views.request_srp, name="request_srp"),
    url(
        r"^srp-link/(\w+)/complete/$", views.complete_srp_link, name="complete_srp_link"
    ),
    # ajax calls
    url(
        # get active srp links
        r"^call/active-srp-links-data/$",
        views.ajax_dashboard_srp_links_data,
        name="ajax_dashboard_srp_links_data",
    ),
    url(
        # get all srp links
        r"^call/active-srp-links-data/all/$",
        views.ajax_dashboard_srp_links_data,
        {"show_all_links": True},
        name="ajax_dashboard_srp_links_all_data",
    ),
    url(
        # get all srp request for the current user
        r"^call/user-srp-requests-data/$",
        views.ajax_dashboard_user_srp_requests_data,
        name="ajax_dashboard_user_srp_requests_data",
    ),
    url(
        # get all srp requests for the current srp link
        r"^call/srp-link/(\w+)/view-srp-requests-data/$",
        views.ajax_srp_link_view_requests_data,
        name="ajax_srp_link_view_requests_data",
    ),
    url(
        # get addition information for the current srp request
        r"^call/srp-link/(\w+)/srp-request/(\w+)/view-additional-information-data/$",
        views.ajax_srp_request_additional_information,
        name="ajax_srp_request_additional_information",
    ),
    url(
        # change the srp payout amount
        r"^call/srp-link/(\w+)/srp-request/(\w+)/change-srp-payout/$",
        views.ajax_srp_request_change_payout,
        name="ajax_srp_request_change_payout",
    ),
    url(
        # change the srp payout amount
        r"^call/srp-link/(\w+)/srp-request/(\w+)/approve/$",
        views.ajax_srp_request_approve,
        name="ajax_srp_request_approve",
    ),
    url(
        # change the srp payout amount
        r"^call/srp-link/(\w+)/srp-request/(\w+)/deny/$",
        views.ajax_srp_request_deny,
        name="ajax_srp_request_deny",
    ),
    url(
        # change the srp payout amount
        r"^call/srp-link/(\w+)/srp-request/(\w+)/remove/$",
        views.ajax_srp_request_remove,
        name="ajax_srp_request_remove",
    ),
]



"""
notifications helper
"""

from django.contrib.auth.models import User

from allianceauth.notifications import notify

from aasrp.app_settings import aa_discordnotify_active, allianceauth_discordbot_active


def send_user_notification(user: User, level: str, title: str, message: str) -> None:
    """
    send notification to user
    this creates a notification in Auth and a PM in Discord when either
    AA-Discordbot or AA Discord Notifications are installed
    :param user:
    :param level:
    :param title:
    :param message:
    """

    notify(
        user=user,
        title=title,
        level=level,
        message=message,
    )

    # send a PM to the user on Discord if allianceauth-discordbot
    # is active and not aa-discordnotify
    if allianceauth_discordbot_active() and not aa_discordnotify_active():
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_direct_message_by_user_id.delay(
            user_pk=user.pk, message_content=message
        )


def send_message_to_discord_channel(
    channel_id: int, message: str, embed: bool = False
) -> None:
    """
    sending a message to a discord channel
    if AA-Discordbot is installed
    :param channel_id:
    :param message:
    :param embed:
    """

    if allianceauth_discordbot_active():
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_channel_message_by_discord_id.delay(
            channel_id, message, embed
        )



"""
some helper functions
so we don't mess up other files too much
"""

from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.models import EveCharacter

from aasrp.helper.eve_images import get_character_portrait_from_evecharacter
from aasrp.models import get_sentinel_user


def get_formatted_character_name(
    character: EveCharacter,
    with_portrait: bool = False,
    with_copy_icon: bool = False,
    portrait_size: int = 32,
    inline: bool = True,
) -> str:
    """
    get character name with alliance and corp ticker
    :param character:
    :param with_portrait:
    :param with_copy_icon:
    :param portrait_size:
    :param inline:
    """

    character_name = character.character_name

    character__corporation_ticker = ""
    if character.corporation_ticker:
        character__corporation_ticker = "[{corporation_ticker}] ".format(
            corporation_ticker=character.corporation_ticker
        )

    character__alliance_ticker = ""
    if character.alliance_ticker:
        character__alliance_ticker = "{alliance_ticker} ".format(
            alliance_ticker=character.alliance_ticker
        )

    character_name_formatted = (
        "<small class='text-muted'>{alliance_ticker}{corporation_ticker}</small>"
        "<br>{character_name}".format(
            alliance_ticker=character__alliance_ticker,
            corporation_ticker=character__corporation_ticker,
            character_name=character_name,
        )
    )

    if with_copy_icon is True:
        character_name_formatted += (
            "<i "
            'class="aa-srp-fa-icon aa-srp-fa-icon-right copy-text-fa-icon far fa-copy" '
            'data-clipboard-text="{character_name}" title="{title}"></i>'.format(
                character_name=character_name,
                title=_("Copy character name to clipboard"),
            )
        )

    return_value = character_name_formatted

    if with_portrait is True:
        line_break = ""
        if inline is False:
            line_break = "<br>"

        character_portrait_html = get_character_portrait_from_evecharacter(
            character=character, size=portrait_size, as_html=True
        )

        return_value = (
            "{character_portrait}{line_break}"
            "<span class='aasrp-character-portrait-character-name'>"
            "{character_name_formatted}"
            "</span>".format(
                character_portrait=character_portrait_html,
                line_break=line_break,
                character_name_formatted=character_name_formatted,
            )
        )

    return return_value


def get_main_for_character(character: EveCharacter) -> EveCharacter:
    """
    get the main character for a given eve character
    :param character:
    """

    return_value = None

    if character.userprofile:
        return_value = character.userprofile.main_character

    return return_value


def get_user_for_character(character: EveCharacter) -> User:
    """
    get the user for a character
    :param character:
    :return:
    """

    if character.userprofile is None:
        return_value = get_sentinel_user()
    else:
        return_value = character.userprofile.user

    return return_value



"""
some helper functions
so we don't mess up other files too much
"""

from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from aasrp.models import AaSrpLink, AaSrpRequest, AaSrpRequestStatus, AaSrpStatus


@login_required
@permission_required(
    "aasrp.basic_access", "aasrp.manage_srp_requests", "aasrp.manage_srp"
)
def get_dashboard_action_icons(request: WSGIRequest, srp_link: AaSrpLink) -> str:
    """
    getting the action buttons for the dashboard view
    :param request:
    :param srp_link:
    """

    actions = ""

    if srp_link.srp_status == AaSrpStatus.ACTIVE:
        button_request_url = reverse("aasrp:request_srp", args=[srp_link.srp_code])
        actions += (
            '<a href="{btn_link}" '
            'class="btn btn-success btn-sm btn-icon-aasrp" '
            'title="{btn_title}">{btn_icon}</a>'.format(
                btn_link=button_request_url,
                btn_icon='<i class="fas fa-hand-holding-usd"></i>',
                btn_title=_("Request SRP"),
            )
        )

    if request.user.has_perm("aasrp.manage_srp") or request.user.has_perm(
        "aasrp.manage_srp_requests"
    ):
        button_view_url = reverse("aasrp:view_srp_requests", args=[srp_link.srp_code])
        actions += (
            '<a href="{btn_link}" '
            'class="btn btn-primary btn-sm btn-icon-aasrp" '
            'title="{btn_title}">{btn_icon}</a><br>'.format(
                btn_link=button_view_url,
                btn_icon='<i class="fas fa-eye"></i>',
                btn_title=_("View SRP Requests"),
            )
        )

        if srp_link.srp_status != AaSrpStatus.COMPLETED:
            if request.user.has_perm("aasrp.manage_srp"):
                if srp_link.srp_status == AaSrpStatus.ACTIVE:
                    button_edit_url = reverse(
                        "aasrp:edit_srp_link", args=[srp_link.srp_code]
                    )
                    actions += (
                        '<a href="{btn_link}" '
                        'class="btn btn-info btn-sm btn-icon-aasrp" '
                        'title="{btn_title}">{btn_icon}</a>'.format(
                            btn_link=button_edit_url,
                            btn_icon='<i class="far fa-newspaper"></i>',
                            btn_title=_("Add/Edit AAR Link"),
                        )
                    )

                    button_disable_url = reverse(
                        "aasrp:disable_srp_link", args=[srp_link.srp_code]
                    )

                    data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                    actions += (
                        '<a class="btn btn-warning btn-sm btn-icon-aasrp" '
                        'title="{btn_title}" '
                        'data-toggle="modal" '
                        'data-target="#{modal_id}" '
                        'data-url="{data_url}" '
                        'data-name="{data_name}">{btn_icon}</a>'.format(
                            data_url=button_disable_url,
                            data_name=data_name,
                            btn_icon='<i class="fas fa-ban"></i>',
                            btn_title=_("Disable SRP Link"),
                            modal_id="disable-srp-link",
                        )
                    )

                if srp_link.srp_status == AaSrpStatus.CLOSED:
                    button_enable_url = reverse(
                        "aasrp:enable_srp_link", args=[srp_link.srp_code]
                    )

                    data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                    actions += (
                        '<a class="btn btn-success btn-sm btn-icon-aasrp" '
                        'title="{btn_title}" '
                        'data-toggle="modal" '
                        'data-target="#{modal_id}" '
                        'data-url="{data_url}" '
                        'data-name="{data_name}">{btn_icon}</a>'.format(
                            data_url=button_enable_url,
                            data_name=data_name,
                            btn_icon='<i class="fas fa-check"></i>',
                            btn_title=_("Enable SRP Link"),
                            modal_id="enable-srp-link",
                        )
                    )

                button_remove_url = reverse(
                    "aasrp:delete_srp_link", args=[srp_link.srp_code]
                )

                data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                actions += (
                    '<a class="btn btn-danger btn-sm btn-icon-aasrp" '
                    'title="{btn_title}" '
                    'data-toggle="modal" '
                    'data-target="#{modal_id}" '
                    'data-url="{data_url}" '
                    'data-name="{data_name}">{btn_icon}</a>'.format(
                        data_url=button_remove_url,
                        data_name=data_name,
                        btn_icon='<i class="far fa-trash-alt"></i>',
                        btn_title=_("Remove SRP Link"),
                        modal_id="delete-srp-link",
                    )
                )

    return actions


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_status_icon(request: WSGIRequest, srp_request: AaSrpRequest) -> str:
    """
    get status icon for srp request
    :param request:
    :param srp_request:
    :return:
    """

    srp_request_status_icon = (
        '<button class="btn btn-info btn-sm btn-icon-aasrp btn-icon-aasrp-status" '
        'title="{request_status_icon_title}">'
        "{request_status_icon}"
        "</button>".format(
            request_status_icon='<i class="fas fa-clock"></i>',
            request_status_icon_title=_("SRP Request Pending"),
        )
    )
    if srp_request.request_status == AaSrpRequestStatus.APPROVED:
        srp_request_status_icon = (
            '<button class="btn btn-success btn-sm btn-icon-aasrp btn-icon-aasrp-status" '
            'title="{request_status_icon_title}">'
            "{request_status_icon}"
            "</button>".format(
                request_status_icon='<i class="fas fa-thumbs-up"></i>',
                request_status_icon_title=_("SRP Request Approved"),
            )
        )

    if srp_request.request_status == AaSrpRequestStatus.REJECTED:
        srp_request_status_icon = (
            '<button class="btn btn-danger btn-sm btn-icon-aasrp btn-icon-aasrp-status" '
            'title="{request_status_icon_title}">'
            "{request_status_icon}"
            "</button>".format(
                request_status_icon='<i class="fas fa-thumbs-down"></i>',
                request_status_icon_title=_("SRP Request Rejected"),
            )
        )

    return srp_request_status_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_details_icon(
    request: WSGIRequest, srp_link: AaSrpLink, srp_request: AaSrpRequest
) -> str:
    """
    get details icon for an SRP request
    :param request:
    :param srp_link:
    :param srp_request:
    """

    button_request_details_url = reverse(
        "aasrp:ajax_srp_request_additional_information",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    srp_request_details_icon = (
        '<button data-link="{link}" '
        'data-toggle="modal" '
        'data-target="#srp-request-details" '
        'data-modal-title="{title}" '
        'data-modal-button-confirm="{modal_button_confirm}" '
        'class="btn btn-primary btn-sm btn-icon-aasrp" '
        'title="{title}">{icon}</button>'.format(
            link=button_request_details_url,
            icon='<i class="fas fa-info-circle"></i>',
            title=_("SRP Request Details"),
            modal_button_confirm=_("{fa_icon} Close".format(fa_icon="")),
        )
    )

    return srp_request_details_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_accept_icon(
    request: WSGIRequest, srp_link: AaSrpLink, srp_request: AaSrpRequest
) -> str:
    """
    get accept icon for an SRP request
    :param request:
    :param srp_link:
    :param srp_request:
    """

    button_request_accept_url = reverse(
        "aasrp:ajax_srp_request_approve",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_accept_state = ""
    if srp_request.request_status == AaSrpRequestStatus.APPROVED:
        button_request_accept_state = ' disabled="disabled"'

    srp_request_accept_icon = (
        '<button data-link="{link}" '
        'data-toggle="modal" '
        'data-target="#srp-request-accept" '
        'data-modal-title="{title}" '
        'data-modal-body="{modal_body}" '
        'data-modal-button-cancel="{modal_button_cancel}" '
        'data-modal-button-confirm="{modal_button_confirm}" '
        'data-modal-button-confirm-classes="{modal_button_confirm_classes}" '
        'class="btn btn-success btn-sm btn-icon-aasrp" '
        'title="{title}"{button_state}>{icon}</button>'.format(
            link=button_request_accept_url,
            icon='<i class="fas fa-check"></i>',
            title=_("Accept SRP Request"),
            modal_body=_("Are you sure you want to accept this SRP request?"),
            modal_button_cancel=_(
                "{fa_icon} Cancel".format(fa_icon="<i class='far fa-hand-paper'></i>")
            ),
            modal_button_confirm=_(
                "{fa_icon} Accept SRP Request".format(
                    fa_icon="<i class='fas fa-check'></i>"
                )
            ),
            modal_button_confirm_classes="btn btn-success btn-sm",
            button_state=button_request_accept_state,
        )
    )

    return srp_request_accept_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_reject_icon(
    request: WSGIRequest, srp_link: AaSrpLink, srp_request: AaSrpRequest
) -> str:
    """
    get reject icon for an SRP request
    :param request:
    :param srp_link:
    :param srp_request:
    """

    button_request_reject_url = reverse(
        "aasrp:ajax_srp_request_deny",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_reject_state = ""
    if srp_request.request_status == AaSrpRequestStatus.REJECTED:
        button_request_reject_state = ' disabled="disabled"'

    srp_request_reject_icon = (
        '<button data-link="{link}" '
        'data-toggle="modal" '
        'data-target="#srp-request-reject" '
        'data-modal-title="{title}" '
        'data-modal-body="{modal_body}" '
        'data-modal-button-cancel="{modal_button_cancel}" '
        'data-modal-button-confirm="{modal_button_confirm}" '
        'data-modal-button-confirm-classes="{modal_button_confirm_classes}" '
        'class="btn btn-warning btn-sm btn-icon-aasrp" '
        'title="{title}"{button_state}>{icon}</button>'.format(
            link=button_request_reject_url,
            icon='<i class="fas fa-ban"></i>',
            title=_("Reject SRP Request"),
            modal_body=_("Are you sure you want to reject this SRP request?"),
            modal_button_cancel=_(
                "{fa_icon} Cancel".format(fa_icon="<i class='far fa-hand-paper'></i>")
            ),
            modal_button_confirm=_(
                "{fa_icon} Reject SRP Request".format(
                    fa_icon="<i class='fas fa-ban'></i>"
                )
            ),
            modal_button_confirm_classes="btn btn-warning btn-sm",
            button_state=button_request_reject_state,
        )
    )

    return srp_request_reject_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_delete_icon(
    request: WSGIRequest, srp_link: AaSrpLink, srp_request: AaSrpRequest
) -> str:
    """
    get delete icon for an SRP request
    :param request:
    :param srp_link:
    :param srp_request:
    """

    button_request_delete_url = reverse(
        "aasrp:ajax_srp_request_remove",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    srp_request_delete_icon = (
        '<button data-link="{link}" '
        'data-toggle="modal" '
        'data-target="#srp-request-remove" '
        'data-modal-title="{title}" '
        'data-modal-body="{modal_body}" '
        'data-modal-button-cancel="{modal_button_cancel}" '
        'data-modal-button-confirm="{modal_button_confirm}" '
        'data-modal-button-confirm-classes="{modal_button_confirm_classes}" '
        'class="btn btn-danger btn-sm btn-icon-aasrp" '
        'title="{title}">{icon}</button>'.format(
            link=button_request_delete_url,
            icon='<i class="fas fa-trash-alt"></i>',
            title=_("Remove SRP Request"),
            modal_body=_("Are you sure you want to remove this SRP request?"),
            modal_button_cancel=_(
                "{fa_icon} Cancel".format(fa_icon="<i class='far fa-hand-paper'></i>")
            ),
            modal_button_confirm=_(
                "{fa_icon} Remove SRP Request".format(
                    fa_icon="<i class='fas fa-trash-alt'></i>"
                )
            ),
            modal_button_confirm_classes="btn btn-danger btn-sm",
        )
    )

    return srp_request_delete_icon


@login_required
@permission_required("aasrp.manage_srp_requests", "aasrp.manage_srp")
def get_srp_request_action_icons(
    request: WSGIRequest, srp_link: AaSrpLink, srp_request: AaSrpRequest
) -> str:
    """
    get action icons for srp requests
    :param request:
    :param srp_code:
    :param srp_request:
    """

    srp_request_action_icons = get_srp_request_details_icon(
        request=request, srp_link=srp_link, srp_request=srp_request
    )

    if (
        srp_link.srp_status == AaSrpStatus.ACTIVE
        or srp_link.srp_status == AaSrpStatus.CLOSED
    ):
        # accept
        srp_request_action_icons += get_srp_request_accept_icon(
            request=request, srp_link=srp_link, srp_request=srp_request
        )

        # reject
        srp_request_action_icons += get_srp_request_reject_icon(
            request=request, srp_link=srp_link, srp_request=srp_request
        )

        # delete
        if request.user.has_perm("aasrp.manage_srp"):
            srp_request_action_icons += get_srp_request_delete_icon(
                request=request, srp_link=srp_link, srp_request=srp_request
            )

    return srp_request_action_icons
